"""Entry point for mimosa package command line interface.

A minimal modern data stack with working data pipelines in a single Docker container.
"""
from loguru import logger

from mimosa.pipelines import GEI


def main():
    """Entry Point.

    Also used as entry point by Google Cloud Function.
    """
    logger.debug("mimosa starting.")

    capture_gas_data(
        destination="motherduck", gas_date=None, to_gas_date=None, reporting_update=True
    )

    logger.debug("mimosa completed.")

    return -1


def capture_gas_data(
    destination="motherduck", gas_date=None, to_gas_date=None, reporting_update=True
):
    """Run the pipeline to obtain and capture gas data.

    With default values the pipeline will request the latest available data. Will store the results in a
    motherduck database. And will run the reporting pipeline.

    If no dates are provided, the pipeline will request the latest available data.

    Parameters:
        destination (str): The database type where to capture the data. Defaults to "motherduck", alternately "duckdb".
        gas_date (datetime): The starting date for requesting the data. Defaults to None.
        to_gas_date (datetime): The ending date for requesting the data. Defaults to None.
        reporting_update (bool): Determines whether to perform reporting updates. Defaults to True.
    """
    pipeline = GEI(destination=destination)
    pipeline.run_landing_pipeline(
        gas_date=gas_date, to_gas_date=to_gas_date, reporting_update=reporting_update
    )


if __name__ == "__main__":
    main()



"""Sample dlt pipeline.

Loads data from European Gas data REST API.

Loads data incrementally into DuckDB.
"""
from datetime import date  # F401

from mimosa.pipelines import GEI

destination = "filesystem"  # "filesystem" "motherduck"
pipeline = GEI(destination=destination)
reporting_update = False

run_this = True
if run_this:
    pipeline.run_landing_pipeline(
        gas_date=date(2023, 9, 27),  # still from 2019-01-01 to 2019-09-01
        to_gas_date=date(2023, 9, 28),
        reporting_update=reporting_update,
    )
    """
    pipeline.run_landing_pipeline(
        reporting_update=reporting_update,
    )
    """
else:
    if destination == "motherduck":
        pipeline.run_landing_pipeline(reporting_update=reporting_update)
    else:
        pipeline.run_landing_pipeline(reporting_update=False)

if not reporting_update and destination == "motherduck":
    pipeline.run_reporting_pipeline()



"""Various random snippets."""

from datetime import date, timedelta
from typing import Iterator, Optional


def daterange(
    start_date: Optional[date] = None, end_date: Optional[date] = None
) -> Iterator[date]:
    """Generates an iterator of dates in a range of dates.

    Args:
        start_date (date): The start date of the range
        end_date (date): The end date of the range

    Yields:
        Iterator[date]: A generator that yields each date in the range

    >>> import datetime
    >>> start_date = datetime.datetime.strptime("2023-08-10", "%Y-%m-%d").astimezone()
    >>> end_date = datetime.datetime.strptime("2023-08-15", "%Y-%m-%d").astimezone()

    >>> for dte in daterange(start_date, end_date):
    ...     print(dte.strftime("%Y-%m-%d"))
    2023-08-10
    2023-08-11
    2023-08-12
    2023-08-13
    2023-08-14

    >>> for dte in daterange(end_date, start_date):
    ...     print(dte.strftime("%Y-%m-%d"))
    2023-08-10
    2023-08-11
    2023-08-12
    2023-08-13
    2023-08-14

    >>> for dte in daterange(start_date):
    ...     print(dte.strftime("%Y-%m-%d"))
    2023-08-10

    >>> for dte in daterange(end_date = end_date):
    ...     print(dte.strftime("%Y-%m-%d"))
    2023-08-14

    >>> for dte in daterange():
    ...     print(dte.strftime("%Y-%m-%d"))

    >>> print([dte.strftime("%Y-%m-%d") for dte in daterange(start_date, start_date)])
    []
    """
    if start_date is None and end_date is None:
        start_date = date(2023, 8, 10)
        end_date = start_date
    elif end_date is None:
        end_date = start_date + timedelta(days=1)
    elif start_date is None:
        start_date = end_date - timedelta(days=1)

    if start_date > end_date:  # reversed dates are accepted. Is this desired?
        start_date, end_date = end_date, start_date

    for n in range(int((end_date - start_date).days)):
        yield start_date + timedelta(n)


def chunker(seq: list, size: int) -> Iterator[list]:
    """Generate an iterator that yields chunks of the sequence with the specified size.

    Args:
        seq (list): The sequence to be chunked.
        size (int): The size of each chunk.

    Yields:
        Iterator[list]: A generator that yields lists of size `size` from `seq`.

    >>> for chunk in chunker([1, 2, 3, 4, 5], 2):
    ...     print(chunk)
    [1, 2]
    [3, 4]
    [5]
    """
    return (seq[pos : pos + size] for pos in range(0, len(seq), size))


class MissingEnvironmentVariableError(Exception):
    """Raised when a required environment variable is missing."""


if __name__ == "__main__":
    import doctest

    doctest.testmod()
